<template>
    <div>
        <section class="games">
            <GameSwitch page="jackpot" />
            <div class="jackpot">
                <div class="spinner-border" v-if="isLoading" style="display: flex; justify-content: center; margin: auto;">
                    <span class="sr-only">Loading...</span>
                </div>
                <div class="jackpot__row" v-else>
                    <div class="flex-35">
                        <div class="input-item">
                            <label for="sum" class="jackpot__label">Сумма</label>
                            <div class="jackpot__group">
                                <input 
                                    class="jackpot__amount" 
                                    type="number" 
                                    min="1" 
                                    max="99999" 
                                    step="1" 
                                    v-model="amount" 
                                />
                                <button type="button" class="jackpot__btn" @click="typeBet('min')">
                                    Min
                                </button>
                                <button type="button" class="jackpot__btn" @click="typeBet('max')">
                                    Max
                                </button>
                                <button type="button" class="jackpot__btn" @click="typeBet('x2')">
                                    X2
                                </button>
                                <button type="button" class="jackpot__btn" @click="typeBet('/2')">
                                    /2
                                </button>
                            </div>
                        </div>
                        <div 
                            class="btn-primary"
                        >
                            <button 
                                type="submit" 
                                class="btn-primary__body"
                                :disabled="btnLoading"
                                @click="createBet"
                            >
                                <div class="spinner-border" v-if="btnLoading">
                                    <span class="sr-only">Loading...</span>
                                </div>
                                <span v-else>Поставить</span>
                            </button>
                        </div>
                        <div class="jackpot__chance mt-1">
                            Ваш шанс:&nbsp;
                            <span style="font-weight: 400">
                                {{ parseFloat(myChance).toFixed(2) }}%
                            </span>
                        </div>
                        <span class="showHistory" @click="showHistory = !showHistory">История игр</span>
                    </div>
                    <div class="flex-60">
                        <span style="font-size: 18px;">
                            <i class="fas fa-trophy" style="color: gold;" aria-hidden="true"></i>
                            Джекпот: <span style="font-weight: 400;">{{ parseFloat(bank).toFixed(2) }}</span>
                        </span>
                        <div class="progress mt-1" style="height: 3rem;position:relative">
                            <div 
                                class="progress-bar progress-bar-striped progress-bar-animated" 
                                role="progressbar" 
                                aria-valuemin="0" 
                                aria-valuemax="100" 
                                :style="[
                                {
                                    'width': timer/20 * 100 + '%'
                                }]"
                            >
                                <span class="jackpot__timer">
                                    <span>
                                        <i class="fa-solid fa-hourglass"></i> {{ timer }}
                                    </span>
                                </span>
                            </div>
                        </div>
                        <div class="jackpot-box mt-1" style="display: none">
                            <div class="jackpot-roll" id="jpRoll" style="will-change: auto; width: 100%; max-width: 100%; display: flex; justify-content: center">
                                <div id="jackpot-arrow"></div>
                                <div class="inbox"></div>
                            </div>
                        </div>
                        <div class="card mt-1" id="shownWinner" style="display: none;background: #f2f3f4">
                            <div class="card-body d-flex" style="justify-content: center;text-align: center">
                                <h4 id="winnerStatus" style="font-weight: 400"></h4>
                            </div>
                        </div>

                        <div class="players-list mt-1">
                            <div class="playerin-list" v-for="player in players">
                                <img :src="player.photo_200">
                                <div 
                                    class="uChance"
                                    :class="[$root.user !== null && $root.user.is_admin ? 'pointer' : '']"
                                    @click="setWinner(player.user_id)"
                                >
                                    {{ parseFloat(player.chance) }}%
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </section>
        <History v-if="showHistory" />
        <section class="" style="margin-top: 16px">
            <table class="table">
                <thead class="table__head">
                    <tr class="table__head-row">
                        <td class="table__head-item">
                            Пользователь
                        </td>
                        <td class="table__head-item">
                            Билеты
                        </td>
                        <td class="table__head-item">
                            Ставка
                        </td>
                    </tr>
                </thead>
                <tbody class="table__body">
                    <tr class="table__body-row" v-for="bet in bets">
                        <td class="table__body-item table__padding">
                            <img 
                                :src="bet.photo_200"
                                style="width: 32px;border-radius: 4px;margin-right: 10px;"
                            >
                            <span 
                                @click="$root.$emit('showProfile', bet.user_id)" 
                                style="cursor: pointer"
                            >
                                {{ bet.username }}
                            </span>
                        </td>
                        <td class="table__body-item">
                            <i class="fa-solid fa-ticket"></i>&nbsp;{{ bet.ticket_from }} - {{ bet.ticket_to }}
                        </td>
                        <td class="table__body-item" style="font-weight: 400">
                            {{ parseFloat(bet.amount).toFixed(2) }}&nbsp;<img src="/assets/img/coins.svg" style="width:22px">
                        </td>
                    </tr>
                </tbody>
            </table>
        </section>
    </div>
</template>

<script>
import $ from "jquery";
import GameSwitch from '../components/GameSwitch'
import History from '../components/JackpotHistory'

export default {
    components: {
        GameSwitch,
        History
    },
    data() {
        return {
            btnLoading: false,
            isLoading: true,
            timer: 20,
            amount: 1,
            players: [],
            bets: [],
            bank: 0,
            myChance: 0,
            showHistory: false
        }
    },
    methods: {
        init() {
            this.$root.axios.post('/jackpot/initGame')
            .then(response => {
                this.isLoading = false
                const {data} = response
                this.players = data.parse
                this.bank = data.bank
                this.bets = data.bets
            })
        },
        setWinner(user_id) {
            if(!this.$root.user.is_admin) return;

            this.$root.axios.post('/jackpot/setWinner', {
                user_id: user_id
            })
            .then(response => {
                const {data} = response

                if(data.success) {
                    return notify({
                        theme: 'green',
                        text: 'Вы подкрутили пользователю'
                    })
                }
            })
        },
        createBet() {
            this.btnLoading = true
            this.$root.axios.post('/jackpot/createBet', {
                amount: this.amount
            })
            .then(response => {
                const {data} = response
                this.btnLoading = false

                if(data.error) {
                    return notify({
                        theme: 'red',
                        text: data.message
                    })
                }

                this.$root.user.balance = data.balance
            })
        },
        updateBalance() {
            this.$root.axios.post('/user/balance')
            .then(response => {
                const {data} = response
                this.$root.user.balance = data.balance
                notify({
                    theme: 'green',
                    text: 'Вы выиграли в Jackpot!'
                })
            })
        },
        typeBet(type) {
            switch (type) {
                case "max":
                    this.amount = this.$root.user !== null ? this.$root.user.balance : 0;
                break;
                case "min":
                    this.amount = 1;
                break;
                case "x2":
                    this.amount = (this.amount * 2).toFixed(2);
                break;
                case "/2":
                    this.amount = (this.amount / 2).toFixed(2);
                break;
            }
        }
    },
    mounted() {
        this.init()
        this.$root.$on('closeJackpotHistory', () => {
            this.showHistory = false
        })
    },
    sockets: {
        jackpot(data) {
            if(data.type == 'bet') {
                this.players = data.parse
                this.bank = data.bank
                this.bets = data.bets
            }
            if(data.type == 'timer') {
                this.timer = data.sec
            }
            if(data.type == 'roll') {
                $('.jackpot-box').slideDown()
                data.list.forEach((item, index) => {
                    $('.jackpot-roll > .inbox').append(`
                        <img 
                            data-id="${index}"
                            src="${item}" 
                            style="min-width: 50px;max-width: 50px;min-height: 50px;max-height: 50px;margin: 5px;border-radius:6px"
                        >
                    `)
                });
                
                $('.jackpot-roll > .inbox').css({
                    transition: 'all 10s cubic-bezier(0, 0, 0, 1) 0ms',
                    marginLeft: -data.ml
                })
            }
            if(data.type == 'result') {
                var $container = $(".inbox");
                var $result = $container.find('[data-id="92"]');
                $result.css('border', '3px solid #4bb543');
                $("#shownWinner").fadeIn();

                var typed = new Typed('#winnerStatus', {
                    strings: ["", `Победил: <span style="font-weight:600">${data.username}</span> с шансом <span style="font-weight:600">${parseFloat(data.chance).toFixed(2)}</span>%<br>Билет: #${data.ticket}`],
                    typeSpeed: 18,
                    backSpeed: 100,
                    onComplete: function(s) { },
                });

                setTimeout(() => {
                    this.bets = []
                    this.players = []
                    this.timer = 20
                    this.myChance = 0
                    this.bank = 0

                    $('.jackpot-box').slideUp()
                    $('.jackpot-roll > .inbox').html('')
                    $('.jackpot-roll > .inbox').css({
                        transition: 'all 0s cubic-bezier(0, 0, 0, 1) 0ms',
                        marginLeft: -80
                    })
                    $("#shownWinner").hide();
                    typed.destroy();

                    if(this.$root.user.id == data.user_id) this.updateBalance()
                }, 5000);
            }
        }
    },
    watch: {
        bets: function() {
            if(this.$root.user === null) return;

            for(let index = 0; index < this.bets.length; index++) {
                if(this.bets[index].user_id == this.$root.user.id) {
                    this.myChance = this.bets[index].chance
                }
            }
        }
    }
}
</script>

<style scoped>
.flex-35 {
    text-align: center
}

.jackpot__chance {
    margin-bottom: 10px
}

.showHistory {
    color: #c3bdde;
    cursor: pointer;
    font-weight: 400;
    text-align: center;
    user-select: none
}
.pointer {
    cursor: pointer;
}
.jackpot__group {
    display: flex;
    flex-wrap: wrap;
    border: 1px solid #6f7cad;
    border-radius: 0 0 6px 6px;
    justify-content: space-between;
    width: 100%;
}

.jackpot__label {
    font-size: 13px;
    line-height: 15px;
    text-align: center;
    letter-spacing: 0.03em;
    display: inline-block;
    color: #b6b0d7;
    margin-bottom: 6px;
    text-transform: uppercase;
}

.jackpot__label_normal {
    text-transform: none;
    font-size: 16px;
    line-height: 25px;
}

.jackpot__amount {
    font-weight: 400;
    font-size: 25px;
    font-family: "Rubik";
    color: #2c3034;
    border: none;
    border-bottom: 1px solid #6f7cad;
    flex: 0 0 100%;
    line-height: 52px;
    width: 100%;
    text-align: left
}

.jackpot__amount,
.jackpot__btn {
    text-align: center;
    letter-spacing: 0.03em;
}

.jackpot__btn {
    cursor: pointer;
    flex: 1 0 15%;
    font-weight: 700;
    font-size: 12px;
    line-height: 14px;
    background: transparent;
    color: #9fa6c5;
    height: 33px;
    display: flex;
    align-items: center;
    justify-content: center;
    border: none;
    padding: 0;
}

.jackpot__btn + .jackpot__btn {
    border-left: 1px solid #6f7cad;
}

.btn {
    border-radius: 3px;
    font-size: 14px;
    line-height: 16px;
    text-align: center;
    letter-spacing: 0.03em;
    color: #fff;
    padding: 15px 0;
    width: 100%;
    border: none;
    cursor: pointer;
    user-select: none;
}

button > .spinner-border {
    max-width: 1rem;
    max-height: 1rem;
}
.inRollavatar {
    border-radius: 6px;
    width: 50px;
    height: 50px;
    margin-top: 5px;
}

.jackpot-roll > .inbox {
    width: 5500px; 
    margin-left: -80px; 
    transition: all 0s cubic-bezier(0, 0, 0, 1) 0ms;
    justify-content: center;
    align-items: center;
    align-content: center;
    display: flex;
}

.table {
    display: table;
}

table {
    min-width: 100%;
    margin-top: 16px;
}
tbody {
    text-align: center;
    margin-top: 10px;
}
td {
    font-size: 18px;
}
.table__padding {
    align-items: center;
    align-content: center;
    justify-content: center;
    display: flex;
}

.jackpot {
    padding: 32px 57px;
    background: #fff;
    border-radius: 0 0 8px 8px;
}

.jackpot__row {
    display: flex;
    flex-direction: row;
    flex-wrap: wrap;
    justify-content: space-between
}

.jackpot__bet {
    background: #eeecfd;
    padding: 18px 21px;
    border: 1px solid #dcd9f7;
    -webkit-box-sizing: border-box;
    box-sizing: border-box;
    -webkit-box-shadow: 0 5px 35px rgba(215, 211, 236, 0.25);
    box-shadow: 0 5px 35px rgba(215, 211, 236, 0.25);
    border-radius: 4px;
    font-size: 13px;
    line-height: 15px;
    width: 100%;
    max-height: 50px;
}

.jackpot__timer {
    font-size: 14px;
}

.jackpot__timer > span {
    position: absolute;
    border-radius: 8px;
    color: #000;
    display: block;
    font-size: 16px;
    background: #fff;
    padding: 4px;
    font-weight: 600;
    top: 50%;
    left: 50%;
    margin-right: -50%;
    transform: translate(-50%, -50%);
    position: absolute;
    width: auto;
}

.jackpot__chance {
    display: flex;
    background: #f2f3f4;
    box-shadow: rgb(30 41 59 / 4%) 0 2px 4px 0;
    border: 1px solid rgba(98, 105, 118, .16);
    min-height: 60px;
    justify-content: center;
    align-items: center;
    border-radius: 6px
}

#jackpot-arrow {
    border: 2px solid #eee;
	background: #fff;
    position: absolute;
    left: calc(50%);
    height: 60px;
    box-shadow: 0px 0px 15px mediumpurple;
    z-index: 99;
}

.jackpot-roll .inbox {
	transform: translate3d(0,0,0);
}

.jackpot-roll {
	background: #f2f3f4 no-repeat;
	width: 100%;
	position: relative;
	height: 60px;
	margin-bottom: 10px;
	text-align: center;
	overflow: hidden;
}

.players-list {
    padding: 6px;
    border-radius: 6px;
    background: #f2f3f4;
    max-width: 100%;
    overflow-y: hidden;
    display: flex;
    flex-direction: row;
}

.playerin-list {
    width: 50px;
}

.playerin-list:not(:first-child) {
    margin-left: 5px
}

.playerin-list img {
    width: 50px;
    background-position: center;
    height: 50px;
    border-top-left-radius: 6px;
    border-top-right-radius: 6px;
}

.playerin-list div {
    background: #fff;
    position: relative;
    text-align: center;
    margin-top: -5px;
    width: 50px;
    font-weight: 600;
    font-size: 10px;
    padding: 5px;
	white-space: nowrap;
    border-bottom-left-radius: 6px;
    border-bottom-right-radius: 6px;
    color: #626976!important;
}

.btn-primary {
    margin-top: 10px;
    min-width: 100%;
}

.mt-1 {
    margin-top: 10px
}

.flex-35 {
    flex: 0 35%
}

.flex-60 {
    flex: 0 60%;
    max-width: 60%;
}

@media (max-width: 62em) {
    .jackpot {
        padding: 12px 15px 23px;
    }

    .flex-35, .flex-60 {
        flex: 0 100%;
        max-width: 100%;
    }

    .flex-35 {
        margin-bottom: 20px
    }
}

@-webkit-keyframes progress-bar-stripes {
	0% {
		background-position-x: .5rem
	}
}

@keyframes progress-bar-stripes {
	0% {
		background-position-x: .5rem
	}
}

.progress {
	display: flex;
	height: .5rem;
	overflow: hidden;
	font-size: .65625rem;
    background: #f2f3f4;
	border-radius: 4px
}

.progress-bar {
	display: flex;
	flex-direction: column;
	justify-content: center;
	overflow: hidden;
	color: #fff;
	text-align: center;
	white-space: nowrap;
	background: #2c80ff;
	transition: width .6s ease
}

@media (prefers-reduced-motion:reduce) {
	.progress-bar {
		transition: none
	}
}

.progress-bar-striped {
	background-image: linear-gradient(45deg, rgba(255, 255, 255, .15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, .15) 50%, rgba(255, 255, 255, .15) 75%, transparent 75%, transparent);
	background-size: .5rem .5rem
}

.progress-bar-animated {
	-webkit-animation: 1s linear infinite progress-bar-stripes;
	animation: 1s linear infinite progress-bar-stripes
}

@media (prefers-reduced-motion:reduce) {
	.progress-bar-animated {
		-webkit-animation: none;
		animation: none
	}
}

.typed-cursor{
	display: none;
}

.spinner-border {
    display: inline-block;
    width: 2rem;
    height: 2rem;
    vertical-align: text-bottom;
    border: .25em solid currentColor;
    border-right-color: currentcolor;
    border-right-color: transparent;
    border-radius: 50%;
    animation: spinner-border .75s linear infinite;
}

@keyframes spinner-border {
    to {
        transform: rotate(360deg)
    }
}
</style>